package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.Gid;
import com.brilliance.isc.bo.Git;
import com.brilliance.isc.bo.lg.GitcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.GitcanStoreVo;
import com.brilliance.isc.vo.model.GidVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitcan Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring"
)
public interface GitcanStructMapper extends IBaseStructMapper<GitcanStoreBo, GitcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GitcanStructMapper INSTANCE = Mappers.getMapper(GitcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GitcanStoreBo voTransferToBo(GitcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mapping(target = "gidgrp", ignore = true)
    @Mapping(target = "oldgidgrp", ignore = true)
    void voUpdateToBo(GitcanStoreVo vo, @MappingTarget GitcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(GitcanStoreVo vo, @MappingTarget GitcanStoreBo bo) {
        bo.getGidgrp().getRec().setPurcan(vo.getGidgrp().getRec().getPurcan());
        bo.getGidgrp().getRec().setClsdat(vo.getGidgrp().getRec().getClsdat());
        bo.getGidgrp().getRec().setReddat(vo.getGidgrp().getRec().getReddat());
        bo.getGidgrp().getRec().setRedcur(vo.getGidgrp().getRec().getRedcur());
        bo.getGidgrp().getRec().setRedamt(vo.getGidgrp().getRec().getRedamt());
        bo.getGidgrp().getRec().setOutcur(vo.getGidgrp().getRec().getOutcur());
        bo.getGidgrp().getRec().setOutamt(vo.getGidgrp().getRec().getOutamt());
        Git git = bo.getGidgrp().getBlk();
        if (git != null) {
            git.setAccspc(vo.getGidgrp().getBlk().getAccspc());
            git.setChaded(vo.getGidgrp().getBlk().getChaded());
            git.setAmtspc(vo.getGidgrp().getBlk().getAmtspc());
        }
    }


    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GitcanStoreVo boTransferToVo(GitcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GitcanStoreBo bo, @MappingTarget GitcanStoreVo vo);
}
