package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.GitdlaStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.GitdlaStoreVo;
import com.brilliance.isc.vo.model.CbbVo;
import com.brilliance.isc.vo.model.Giacbs;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitopn Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface GitdlaStructMapper extends IBaseStructMapper<GitdlaStoreBo, GitdlaStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GitdlaStructMapper INSTANCE = Mappers.getMapper(GitdlaStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GitdlaStoreBo voTransferToBo(GitdlaStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldgidgrp",ignore = true),
            @Mapping(target = "gidgrp",ignore = true)
    })
    void voUpdateToBo(GitdlaStoreVo vo, @MappingTarget GitdlaStoreBo bo);
    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GitdlaStoreVo boTransferToVo(GitdlaStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GitdlaStoreBo bo, @MappingTarget GitdlaStoreVo vo);
}
