package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.GitfreStoreBo;
import com.brilliance.isc.bo.lg.GitpfrStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.GitfreStoreVo;
import com.brilliance.isc.vo.lg.GitpfrStoreVo;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Gitpfr Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface GitfreStructMapper extends IBaseStructMapper<GitfreStoreBo, GitfreStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    GitfreStructMapper INSTANCE = Mappers.getMapper(GitfreStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    GitfreStoreBo voTransferToBo(GitfreStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "gidgrp",ignore = true)
    })
    void voUpdateToBo(GitfreStoreVo vo, @MappingTarget GitfreStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    GitfreStoreVo boTransferToVo(GitfreStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(GitfreStoreBo bo, @MappingTarget GitfreStoreVo vo);
}
