package com.brilliance.isc.transfer.transaction;

import com.brilliance.mda.runtime.mda.util.Systems;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.HitameStoreVo;
import com.brilliance.isc.bo.lg.HitameStoreBo;
import com.brilliance.isc.vo.model.CbbVo;
import com.brilliance.isc.vo.model.Giacbs;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface HitameStructMapper extends IBaseStructMapper<HitameStoreBo, HitameStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    HitameStructMapper INSTANCE = Mappers.getMapper(HitameStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    HitameStoreBo voTransferToBo(HitameStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldnidgrp",ignore = true),
            //修改交易，两个机构记账机构和所属机构忽略赋值
            @Mapping(target = "nidgrp.rec.branchinr",ignore = true),
            @Mapping(target = "nidgrp.rec.bchkeyinr",ignore = true)
    })
    void voUpdateToBo(HitameStoreVo vo, @MappingTarget HitameStoreBo bo);

    @AfterMapping
    default void afterUpdate(HitameStoreVo vo, @MappingTarget HitameStoreBo bo){
        Giacbs cbs = vo.getNidgrp().getCbs();
        if(cbs == null) return;
        CbbVo max = cbs.getMax();
        if(max == null) return;
        bo.getNidgrp().setAmtwrdtxtz(Systems.moneycnsmalltobig(String.valueOf(max.getAmt())));
        bo.getNidgrp().setFmtamtz(String.valueOf(max.getAmt()));
    }
    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    HitameStoreVo boTransferToVo(HitameStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(HitameStoreBo bo, @MappingTarget HitameStoreVo vo);
}
