package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NetcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NetcanStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NetcanStructMapper extends IBaseStructMapper<NetcanStoreBo, NetcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NetcanStructMapper INSTANCE = Mappers.getMapper(NetcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NetcanStoreBo voTransferToBo(NetcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({@Mapping(target = "nidgrp", ignore = true)})
    void voUpdateToBo(NetcanStoreVo vo, @MappingTarget NetcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(NetcanStoreVo vo, @MappingTarget NetcanStoreBo bo) {
        bo.getNidgrp().getRec().setPurcan(vo.getNidgrp().getRec().getPurcan());
        bo.getNidgrp().getRec().setClsdat(vo.getNidgrp().getRec().getClsdat());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NetcanStoreVo boTransferToVo(NetcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NetcanStoreBo bo, @MappingTarget NetcanStoreVo vo);
}
