package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NitfeeStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NitfeeStoreVo;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NitfeeStructMapper extends IBaseStructMapper<NitfeeStoreBo, NitfeeStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NitfeeStructMapper INSTANCE = Mappers.getMapper(NitfeeStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NitfeeStoreBo voTransferToBo(NitfeeStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "nidgrp",ignore = true)
    })
    void voUpdateToBo(NitfeeStoreVo vo, @MappingTarget NitfeeStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NitfeeStoreVo boTransferToVo(NitfeeStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NitfeeStoreBo bo, @MappingTarget NitfeeStoreVo vo);
}
